import uuid

from logger.src.common.enums.platform import Platform

class Payload:
    def __init__(self, package, eventTime, message, tags, detail):
        self.eventId = str(uuid.uuid4())
        self.package = package
        self.platform = Platform.python
        self.eventTime = eventTime
        self.message = message
        self.tag = tags
        self.detail = detail



import logger.src.common.dto.payload
import logger.src.common.dto.payloadDetail
import logger.src.common.dto.payloadDetailSdk
import logger.src.common.dto.payloadDetailMetadata
import logger.src.common.dto.user
import logger.src.common.dto.userGeo



class PayloadDetailMetadata:
    def __init__(self, filename, function, type, value):
        self.filename = filename
        self.function = function
        self.type = type
        self.value = value


import pathlib
import setuptools

from distutils.core import setup

from logger.src.version import __version__

HERE = pathlib.Path(__file__).parent

README = (HERE / 'README.md').read_text()

setup(
    name='alertnow-python',  # How you named your package folder (MyLib)
    packages=setuptools.find_packages(),  # Chose the same as "name"
    version=__version__,  # Start with a small number and increase it with every change you make
    license='MIT',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    long_description=README,
    long_description_content_type='text/markdown',
    description='This package used for logging information and errors.',  # Give a short description about your library
    author='Tabriz Gulmammadov',  # Type in your name
    author_email='gulmammadovtabriz@gmail.com',  # Type in your E-Mail
    url='https://gitlab.com/log-collector/alertnow-phyton',  # Provide either the link to your github or to your website
    download_url='https://gitlab.com/log-collector/alertnow-phyton/-/archive/main/alertnow-phyton-main.tar.gz',
    # I explain this later on
    keywords=['Log', 'Logger', 'AlertNow'],  # Keywords that define your package best
    install_requires=[  # I get to this in a second
        'requests'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',  # Again, pick a license
        'Programming Language :: Python :: 3',  # Specify which python versions that you want to support
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    py_modules=[
        'logger',
        'logger.src',
        'logger.src.common',
        'logger.src.common.configuration',
        'logger.src.common.dto',
        'logger.src.common.enums'
    ],  # Name of the python package
)



import json
import sys
import requests
from datetime import datetime

import pkg_resources

from logger.src.common.configuration.httpClientConfig import HttpClientConfig
from logger.src.common.dto.payload import Payload
from logger.src.common.dto.payloadDetail import PayloadDetail
from logger.src.common.dto.payloadDetailMetadata import PayloadDetailMetadata
from logger.src.common.dto.payloadDetailSdk import PayloadDetailSdk
from logger.src.common.enums.logLevel import LogLevel
from logger.src.version import __version__

with requests.Session() as session:
    session.headers.update({
        'Content-Type': 'application/json',
    })

    user_api_key = None
    user_info = None
    tags = []

    def set_api_key(api_key):
        user_api_key = api_key

        session.headers.update({
            'Authorization': f'X-API-KEY {user_api_key}'
        })

    def set_user(user):
        user_info = user

    def set_tag(property_name, property_value):
        tags.append({property_name: property_value})

    def info(message, event_time_required = False):
        installed_packages = [{d.project_name: d.version} for d in pkg_resources.working_set]
        print('installed_packages => ', installed_packages)

        eventTime = datetime.now().strftime("%m/%d/%Y, %H:%M:%S") if event_time_required else None
        print('eventTime => ', eventTime)

        sourceFilePath = sys._getframe().f_code
        print('sourceFilePath => ', sourceFilePath)

        methodName = sys._getframe(1).f_code.co_name
        print('methodName => ', methodName)

        print('version => ', __version__)

        detail = [
            PayloadDetail(
                LogLevel.info,
                sourceFilePath,
                None,
                None,
                None,
                None,
                message,
                LogLevel.info,
                __version__,
                PayloadDetailMetadata(
                    sourceFilePath,
                    methodName,
                    LogLevel.info,
                    message
                ),
                PayloadDetailSdk(
                    __version__,
                    installed_packages
                ),
                user_info
            )
        ]

        return create_log(Payload(
            sourceFilePath,
            eventTime,
            message,
            json.dumps(vars(tags)),
            json.dumps(vars(detail))
        ))

    def error(occurred_error, event_time_required = False):
        message = json.dumps(vars(occurred_error))

        installed_packages = [{d.project_name: d.version} for d in pkg_resources.working_set]
        print('installed_packages => ', installed_packages)

        eventTime = datetime.now().strftime("%m/%d/%Y, %H:%M:%S") if event_time_required else None
        print('eventTime => ', eventTime)

        sourceFilePath = sys._getframe().f_code
        print('sourceFilePath => ', sourceFilePath)

        methodName = sys._getframe(1).f_code.co_name
        print('methodName => ', methodName)

        print('version => ', __version__)

        detail = [
            PayloadDetail(
                LogLevel.error,
                sourceFilePath,
                None,
                None,
                None,
                None,
                message,
                LogLevel.error,
                __version__,
                PayloadDetailMetadata(
                    sourceFilePath,
                    methodName,
                    LogLevel.error,
                    message
                ),
                PayloadDetailSdk(
                    __version__,
                    installed_packages
                ),
                user_info
            )
        ]

        return create_log(Payload(
            sourceFilePath,
            eventTime,
            message,
            json.dumps(tags),
            json.dumps(vars(detail))
        ))

    def create_log(payload):
        response = session.post(f'{HttpClientConfig.alertNowURL}/integration/appinsight/v1/{user_api_key}',
                                data=json.dumps(vars(payload)))
        return response



class PayloadDetail:
    def __init__(
            self,
            logLevel,
            location,
            logger,
            nodestoreInsert,
            received,
            timestamp,
            title,
            type,
            version,
            metadata,
            sdk,
            user):
        self.level = logLevel
        self.location = location
        self.logger = logger
        self.nodestoreInsert = nodestoreInsert
        self.received = received
        self.timestamp = timestamp
        self.title = title
        self.type = type
        self.version = version
        self.metadata = metadata
        self.sdk = sdk
        self.user = user
